<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Tirzen Framework (TZN)
 *
 * This declares the common class from which any object from the TZN shall
 * inherit. It is compatible with PHP versions 4 and 5.
 *
 * THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * This package is licensed under the LGPL License
 * Copyright (C) 2005 Stan Ozier
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * @package    Tirzen Framework MySQL database
 * @author     Stan Ozier <stan@tirzen.net>
 * @copyright  2005 - Stan Ozier
 * @license    http://www.gnu.org/licenses/lgpl.txt (LGPL)
 * @link       http://www.tirzen.net/tzn/
 * @version    1.0
 */
 
/**
 * TZN: Tirzen Framework (TZN) (mysql) DB class
 *
 * @package    TZN-mySQL
 * @author     Stan Ozier <stan@tirzen.net>
 * @version    1.0
 */



/* ------------------------------------------------------------------------ *\
define("TZN_DB_DEBUG",0);
	// 0 no debug, 1 on error, 2 show most, 3 show all
define("TZN_DB_HOST","localhost");
define("TZN_DB_USER","test");
define("TZN_DB_PASS","test");
define("TZN_DB_BASE","test");
define("TZN_DB_PERMANENT",1);
\* ------------------------------------------------------------------------ */

/**
 * DB Connection
 * connects to mySQL database
 * @author   Stan Ozier <stan@tirzen.net>
 * @package  TZN-mySQL
 */

class TznDbConnection {
	var $_dbHost;
	var $_dbUser;
	var $_dbPass;
	var $_dbBase;
	var $_dbLink;

	function TznDbConnection($host=null, $user=null, $pass=null, $base=null) {
        if ($host && $user && $base) {
            $this->_dbHost = $host;
            $this->_dbUser = $user;
            $this->_dbPass = $pass;
            $this->_dbBase = $base;
        } else {
            $this->_dbHost = TZN_DB_HOST;
            $this->_dbUser = TZN_DB_USER;
            $this->_dbPass = TZN_DB_PASS;
            $this->_dbBase = TZN_DB_BASE;
        }
	}

	function connect() {
		if (!$this->_dbLink) {
			if (@constant('TZN_DB_PERMANENT')) {
				$this->_dbLink = @mysql_pconnect($this->_dbHost,$this->_dbUser
					,$this->_dbPass);
			} else {
				$this->_dbLink = @mysql_connect($this->_dbHost,$this->_dbUser
					,$this->_dbPass);
			}
			if (!$this->_dbLink) {
				if (defined("TZN_DB_ERROR_PAGE") && 
					(constant("TZN_DB_ERROR_PAGE")))
				{
	                die('<html><head><meta http-equiv="refresh" content="1;URL='.TZN_DB_ERROR_PAGE.'"></head></html>');
				} else {
					die('cannot connect to Database');
				}
			}
			if (!@mysql_select_db($this->_dbBase,$this->_dbLink)) {
                if (defined("TZN_DB_ERROR_PAGE") && 
                	(constant("TZN_DB_ERROR_PAGE")))
                {
	                die('<html><head><meta http-equiv="refresh" content="1;URL='
	                	.DB_ERROR_PAGE.'"></head></html>');
				} else {
					die('Database does not exist');
				}
			}
		}
	}

	function isConnected() {
		if ($this->_dbLink) {
			return true;
		} else {
			return false;
		}
	}

    function querySelect($qry) {
		return new TznDbResult($qry,mysql_query($qry,$this->_dbLink));
	}

	function queryAffect($qry) {
		if ($this->isConnected()) {
			mysql_query($qry,$this->_dbLink);
			if (($affected_row = mysql_affected_rows($this->_dbLink)) == -1) {
				switch(TZN_DB_DEBUG) {
				case 3:
                case 2:
					echo "<code>".htmlspecialchars($qry)."</code><br/>";
				case 1:
					echo "Error SQL #".mysql_errno().": ".mysql_error()."<br/>";
                    break;
                default:
                    if (defined("TZN_DB_ERROR_PAGE") &&
                    	(constant("DB_ERROR_PAGE"))) {
						die('<html><head><meta http-equiv="refresh" content="1;'
							.'URL='.TZN_DB_ERROR_PAGE.'"></head></html>');
					} else {
						die('Database error (SQL insert)');
					}
                    break;
				}
				// -DEBUG-
				exit;
			} else if (TZN_DB_DEBUG == 3) {
				echo "<code>".htmlspecialchars($qry)."</code><br/>";
			}
			return $affected_row;
		} else {
			echo ("not connected to database");
			exit;
			return false;
		}
	}
}

/**
 * DB Result
 * represents a mySQL result set
 * @author   Stan Ozier <stan@ozier.net>
 * @package  DB
 */

class TznDbResult {
    var $_dbResult;
    var $_count;
    var $_idx;

    function TznDbResult($qry,$result = null) {
        if ($result) {
        	if (TZN_DB_DEBUG == 3) {
        		echo "<code>".htmlspecialchars($qry)."</code><br/>";
        	}
            $this->_dbResult = $result;
            $this->_count = mysql_num_rows($result);
            $this->_idx = 0;
			return $this->_count;
        } else {
			switch(TZN_DB_DEBUG) {
			case 3:
            case 2:
	            echo "<code>".htmlspecialchars($qry)."</code><br/>";
			case 1:
				echo "Error SQL #".mysql_errno().": ".mysql_error()."<br/>";
                break;
            default:
                if (defined("TZN_DB_ERROR_PAGE") 
                	&& (constant("TZN_DB_ERROR_PAGE"))) 
                {
					die('<html><head><meta http-equiv="refresh" content="1;URL='
						.TZN_DB_ERROR_PAGE.'"></head></html>');
				} else {
					die('Database error (SQL general)');
				}
                exit;
                break;
            }
            return false;
        }
    }

    function rCount() {
        return $this->_count;
    }

    function rNext() {
        $row = mysql_fetch_object($this->_dbResult);
        if (!$row) {
			// $this->freeResult();
			return false;
		}
		$this->_idx++;
		return $row;
	}

	function rSkip($num = 1) {
		$next = $this->_idx + $num;
		if (($next < $this->_idx) && (mysql_data_seek($this->_dbResult, $next))) 
		{
			$this->_idx += $num;
			return true;
		} else {
			return false;
		}
	}

	function rItem($num) {
		if (($num < $this->_idx) && (mysql_data_seek($this->_dbResult, $num))) {
			$this->_idx = $num;
			return $this->rNext();
		} else {
			return false;
		}
	}

	function rReset() {
		if (@mysql_data_seek($this->_dbResult, 0)) {
			$this->_idx = 0;
			return true;
		} else {
			return false;
		}
	}

    function rMore() {
        if ($this->_idx < $this->_count) {
            return true;
        } else {
            return false;
        }
    }

    function rFree() {
		if ($this->_dbResult) {
			mysql_free_result($this->_dbResult);
          }
        $this->_idx = 0;
        $this->_count = 0;
      }
}


/**
 * DB Object
 * every object representing data stored in mySQL
 * shall inherit from this class
 * @author   Stan Ozier <stan@ozier.net>
 * @package  DB
 */

class TznDb extends Tzn {

	var $_dbConnection;
	var $_table;
	var $_prevItemId;
	var $_nextItemId;
	var $_data;
	var $_loaded;
	var $_total;
	var $_page;
	var $_pageSize;
	var $_sqlWhere;
	var $_sqlGroup;
	var $_sqlOrder;
    
    function TznDb($table) {
        $this->Tzn();
        $this->_table = $table;
        $this->_page = 1;
    }
	
	function gTable() {
		return (@constant('TZN_DB_PREFIX')?TZN_DB_PREFIX.'_':'').$this->_table;
	}
    
    function setPagination($pageSize,$page=1) {
    	$this->_pageSize = intval($pageSize);
    	$this->_page = intval($page);
    }
    
   	function getIdKey() {
		return $this->_table."Id";
    }

    function getConnection($host=null, $user=null, $pass=null, $base=null) {
        if (!is_object($this->_dbConnection)) {
           $this->_dbConnection = new TznDbConnection($host,$user,$pass,$base);
        }
		$this->_dbConnection->connect();
        return $this->_dbConnection;
    }
    
    function query($qry) {
		global $pSqlQueryCount;
		$pSqlQueryCount++;
		if (DB_DEBUG_LEVEL == 3) {
			echo "<code>".$qry."</code><br/>";
		}
		if (is_object($this->_dbConnection)) {
			$pos = strpos(ltrim($qry),"SELECT");
			if (($pos !== false) && ($pos == 0)) {
				return $this->_dbConnection->querySelect($qry);
			} else {
				return $this->_dbConnection->queryAffect($qry);
			}
		} else {
			echo "no active connection to database - please call $db-&gt;getConnection first";
		}
    }
    
    function isLoaded() {
    	return $this->_loaded;
    }

    /* -- Query Generator Methods  -------------------------------------- */
    
    function getObjId($key) {
      $obj = $this->$key;
      return $obj->id;
    }
    
    function addWhere($data, $sep = 'AND') {
		$this->_sqlWhere = $this->concatSQL($this->_sqlWhere, $data, $sep);
    }
    
    function sqlWhere($default = '') {
		return $this->concatSQL($this->_sqlWhere, $default, 'AND', ' WHERE');
    }
    
    function addOrder($data,$default = '') {
    	if ((!ereg('^[a-zA-Z0-9_\-\., ]+$',$data)) || (empty($data))) {
    		$data = $default;
    	}
    	$this->_sqlOrder = $this->concatSQL($this->_sqlOrder, $data,',');
    }
    
    function sqlOrder($default = '') {
    	return $this->concatSQL($this->_sqlOrder, $default, ',', ' ORDER BY');
    }
    
    function addGroup($data) {
    	$this->_sqlGroup = $this->concatSQL($this->_sqlGroup, $data, ',');
    }
    
    function sqlGroup($default = '') {
    	return $this->concatSQL($this->_sqlGroup,$default, ',',' GROUP BY');
    }
    
    function sqlLimit() {
    	// mySQL only
		if (!$this->_pageSize) {
			return false; // no paging
		}
		if (!$this->_page) {
			$this->_page = 1;
		}
		while ((($this->_page - 1) * $this->_pageSize) > $this->_total) {
			$this->_page--;
		}
		return ' LIMIT '.($this->_page - 1) * $this->_pageSize.', '
			.$this->_pageSize;
	}
    
    /* -- Queries -------------------------------------------------------- */

    function load() {
        // load by Id
		if ($this->id) {
	        return $this->loadByKey($this->getIdKey(), $this->id);
		} else {
			return false;
		}	
		/* else {
			return $this->loadByFilter("1 LIMIT 0,1");
		} */
    }

    function loadByKey($key, $value=null) {
	  if ($value == null) {
		  $value = $this->get($key);
	  }
      return $this->loadByFilter($this->gTable().".".$key." = '".$value."'");
    }

    function loadByFilter($filter) {
    	$this->_data = null;
    	$this->_total = 0;
		if (empty($this->_sqlLoad)) {
			$strSql = 'SELECT * FROM '.$this->gTable();
		} else {
			$strSql = $this->_sqlLoad;
		}
		if (!empty($filter)) {
			$strSql .= " WHERE ".$filter;
			$this->getConnection();
			if ($result = $this->query($strSql)) {
				if ($data = $result->rNext()) {
					$this->setAuto($data);
					$this->_loaded = true;
					if ($this->id) {
						return $this->id;
					} else {
						return true;
					}
				}
			}
		}
		$this->_loaded = false;
		return false;
    }
    
    /* -- Loading List --------------------------------------------- */

    function loadList($sqlSelect,$limit=NULL) {
        
        $this->_data = null;
        $this->_loaded = false;
        $this->_total = 0;

		$sqlSelect .= $this->sqlWhere();	// WHERE statement
		
		$sqlSelect .= $this->sqlGroup();   // GROUP BY statement

		$sqlSelect .= $this->sqlOrder();	// ORDER BY statement

		if ($limit) {
			$sqlSelect .= ' LIMIT '.$limit;
		}

		$this->getConnection();
		
		// echo $sqlSelect; exit;
		
		if ($this->_data = $this->query($sqlSelect)) {
			$return = $this->_data->_count;
		}
		
		if (!$this->_total) {
			$this->_total = $return;
		}
		
		return $return;

    }
    
    /* -- Retreiving -------------------------------------------- */
    
    function rMore() {
    	if ($this->_data) {
	    	return $this->_data->rMore();
	    }
	    return false;
    }
    
    function rCount() {
    	if ($this->_data) {
    		return $this->_data->rCount();
    	}
    	return false;
    }
    
    function rTotal() {
    	return $this->_total;
    }
    
    function rNext() {
		if ($this->rMore()) {
			return $this->_setItem($this->_data->rNext());
		} else {
			return false;
		}
	}
	
	function rSkip($num = 1) {
		if ($this->_data) {
			return $this->_data->rSkip($num);
		}
		return false;
	}
	
	function rItem($num) {
		if ($this->_data) {
			return $this->_setItem($this->_data->rItem($num));
		}
		return false;	
	}
	
	function rReset() {
		return $this->_data->rReset();
	}
	
	function _setItem($obj) {
		if ($obj) {
			$className = get_class($this);
			if (strtolower(get_class($this->_data)) == "tznstaticresult") {
				return $obj;
			} else {
				$objTmp = new $className();
				$objTmp->setAuto($obj);
				return $objTmp;
			}
		} else {
			return false;
		}
	}
    
    /* -- Item functions ---------------------------------------- */

    function add() {
    	// create SQL statement
		$strSql = "INSERT INTO ".$this->gTable()." SET ";
		if ($this->id) {
			$strSql .= $this->getIdKey()."='".$this->id."', ";
		}
		$strSql .= $this->zPropsToSql();
		$this->getConnection();
		if ($this->query($strSql)) {
			if (!$this->id) {
				$this->id = mysql_insert_id();
			}
			return $this->id;
		} else {
			return false;
		} 
    }

    function update($fields=null,$filter=null) {
    	// echo "<pre>--- UPDATE ---\r\n"; $this->dump(); echo "---\r\n";
        $strSql = "UPDATE ".$this->gTable()." SET ";
        if ($fields) {
        	$arrFields = explode(',',$fields);
        	$first = true;
        	forEach($arrFields as $field) {
				$field=trim($field);
				if ($first) {
					$first = false;
				} else {
					$strSql .= ",";
				}
				$strSql .= '`'.$field.'` = '.$this->zValueToSql($this->$field);
			}
		} else {
			$strSql .= $this->zPropsToSql();
		}
		if ($filter) {
        	$strSql .= ' WHERE '.$filter;
        } else if ($this->id) {
			$strSql .= " WHERE ".$this->getIdKey()."='".$this->id."'";
		} else {
			// update everything?!?
			$strSql .= " WHERE 1";
		}
		//echo $strSql; exit; // ."\r\n</pre>";
        return $this->query($strSql);
    }

    function delete($filter = null) {
    	// remove from Database
		$strSql = "DELETE FROM ".$this->gTable()." WHERE ";
		if (!empty($filter)) {
			$strSql .= $filter;
		} else if ($this->id) {
	        $strSql .= $this->getIdKey()."='".$this->id."'";
		} else {
			return false;
		}
        $this->getConnection();
        return $this->query($strSql);
    }

	function emptyTable() {
		$this->getConnection();
		return $this->query('TRUNCATE '.$this->gTable());
	}

    function checkUnique($key, $value, $field=null) {
		if ($value == "") {
			return false;
		}
        $idKey = $this->getIdKey();
        $strSql  = "SELECT ".$idKey.", ".$key;
        if ($field) {
            $strSql .=", ".$field;
        }
        $strSql .= " FROM " .$this->gTable()
            ." WHERE ".$key." = '".$value."'";
        if ($this->id) {
		    $strSql.=" AND ".$idKey." <> ".$this->id;
        }
        $this->getConnection();
        if ($result = $this->query($strSql)) {
            if ($row = $result->rNext()) {
            	// $this->setAuto($row);
                return $row->$idKey;
            }
        }
        return false;
    }

    function concatSQL($begin, $end, $separator = 'AND', $instruction = '') {
		$separator = ' '.trim($separator).' ';
		if (!empty($instruction)) {
			$instruction .= ' ';
		}
		if (empty($begin)) {
			if (empty($end)) {
				return false;
			} else {
				return $instruction.$end;
			}
		} else if (empty($end)) {
			return $instruction.$begin;
		} else {
			return $begin.$separator.$end;
		}
	}

    /* function setSqlProperties($objData, $nested = false) {
        $this->_setObjectProperties("setSql",$objData);
    } */

    function zPropsToSql() {
        // to create SQL statement (insert or update)
        // checks if word is reserved or add ''
        $strSql = "";
        $first = true;
        foreach(get_object_vars($this) as $key => $value) {
            if ($key == "id" || ereg("^_",$key) || ereg("(Count|Last)$",$key)) {
                continue;
            } else if (is_object($value)) {
            	if (is_a($value, 'TznFile')) {	
            		$value->save();
            		$oKey = $key;
            		$oValue = '\''.$value->folder.$value->fileName.'\'';
            		// remove from session
            		$tmpKey = $key.'_tmp';
            		unset($_SESSION[$tmpKey]);
            	} else {
					// $oKey = $value->getIdKey();
					$oKey = $key.'Id';
					$oValue = '\''.$value->id.'\'';
				}
            } else {
                $oKey = $key;
                $oValue = $this->zValueToSql($value);
            }
            if ($first) {
                $first = false;
            } else {
                $strSql.=", ";
            }
			$strSql .= '`'.$oKey.'`='.$oValue;
        }
        return $strSql;
    }
    
    function zValueToSql($value) {
    	 if (is_object($value)) {
			if (is_a($value, 'TznFile')) {
				$value->save();
				$str = '\''.$value->folder.$value->filename.'\'';
			}
		} else if (ereg("^(ENCODE|ENCRYPT|MD5|NOW)\(.*\)", $value)) {
            $str = $value;
        } else {
            $str = "'".addslashes($value)."'";
        }
        return $str;
    }
    
    /* ----- List Order ------ */
    
    function pSort($link, $param, $field, $style='',
    	$imgAsc = TZN_DB_ASC_OFF, $imgAscActive = TZN_DB_ASC_ON, 
    	$imgDesc = TZN_DB_DESC_OFF, $imgDescActive = TZN_DB_DESC_ON) 
    {
		$begin = '<a href="'.$link;
		if (ereg('\?',$link)) {
			$begin .= '&';
		} else {
			$begin .= '?';
		}
		$begin.= $param.'=';
		$end = '"';
		$end .= Tzn::_style($style);
		$end .= '>';
		if (ereg('^'.$field.' ASC',$this->_sqlOrder)) {
            print '<img src="'.$imgAscActive.'">';
		} else {
            print $begin.$field.'+ASC'.$end.'<img src="'.$imgAsc
            	.'" border="0"></a>';
		}
        if (ereg('^'.$field.' DESC',$this->_sqlOrder)) {
			print '<img src="'.$imgDescActive.'">';
		} else {
			print $begin.$field.'+DESC'.$end.'<img src="'.$imgDesc
				.'" border="0"></a>';
		}
	}

}

?>